StaminaCost = {}
StaminaCost.Move = 1
StaminaCost.OpenDoor = 2
StaminaCost.CloseDoor = 2
StaminaCost.Observe = 1

Hero = {}
Hero.mCurrentMove = 1
Hero.mCurrentMoveIndex = 1
Hero.mCurrentTry = 1
Hero.mNextMove = {
   { 2, 4, 3 },
   { 3, 1, 4 },
   { 4, 2, 1 },
   { 1, 3, 2 }
}
Hero.mOppositeMove = { 3, 4, 1, 2 }
Hero.mStamina = 25
Hero.mX = 0
Hero.mY = 0
Hero.mPrevMoveDirection = 1

Rooms = {}

-------------------------------
------ AnalyzeRoom State ------
-------------------------------

AnalyzeRoom = {}
AnalyzeRoom.mFinished = false
AnalyzeRoom.Init = function()
    AnalyzeRoom.mFinished = false
    AnalyzeRoom.mRoomKey = Helper.PositionToKey(Hero.mX, Hero.mY)
end
AnalyzeRoom.Run = function()
    oret = 0
    odirection = 1
    
    message = ""
    
    -- current room
    room = Rooms[AnalyzeRoom.mRoomKey]
    if (room == nil) then
        Helper.InitializeRoom(AnalyzeRoom.mRoomKey)
        room = Rooms[AnalyzeRoom.mRoomKey]
    end
    message = message .. AnalyzeRoom.mRoomKey .. "$"
    
    -- try to observe as many directions as possible with
    -- current stamina
    while ((Hero.mStamina >= StaminaCost.Observe) and (oret == 0) and (odirection <= 4) and (not AnalyzeRoom.mFinished)) do
        oret, obs = per.observe(odirection)
        Hero.mStamina = Hero.mStamina - StaminaCost.Observe
        
        -- store doors in room
        numDoors = #(obs.doors)
        for doorIndex = 1, numDoors do
            door = obs.doors[doorIndex]
            doorX = Hero.mX + door.position[0]
            doorY = Hero.mY + door.position[1]
            
            -- check if room has been already stored
            if (not Helper.IsDoorKnown(AnalyzeRoom.mRoomKey, doorX, doorY)) then
                roomIndex = room.mNumDoors + 1
                room.mDoors[roomIndex] = {}
                roomDoor = room.mDoors[roomIndex]
                if (roomDoor == nil) then
                    Helper.InitializeDoor(AnalyzeRoom.mRoomKey, roomIndex)
                    roomDoor = room.mDoors[roomIndex]
                end
                
                roomDoor.mX = doorX
                roomDoor.mY = doorY
                roomDoor.mVisited = false
                
                room.mNumDoors = room.mNumDoors + 1
                
                message = message .. tostring(room.mNumDoors) .. "," .. tostring(doorX) .. "," .. tostring(doorY) .. "$"
            end
        end
        
        -- increase the direction in which to observe and
        -- check if all directions have been observed
        odirection = odirection + 1
        if (odirection == 5) then
            AnalyzeRoom.mFinished = true
        end
    end
    
    if (AnalyzeRoom.mFinished) then
        -- Determine which door to move next
        for doorIndex = 1, room.mNumDoors do
            if (not room.mDoors[doorIndex].mVisited) then
                message = message .. "e" .. tostring(doorIndex)
            end
        end
    end

    per.think(message)
end

------------------------------
------ Helper functions ------
------------------------------

Helper = {}
Helper.PositionToKey = function(x, y)
    return tostring(x) .. ":" .. tostring(y)
end
Helper.KeyToPosition = function(key)
    x = -1
    y = -1
    
    -- find position of the "-" character
    dashIndex = string.find(key, ":", 1, true)
    if (dashIndex == nil) then
        return x, y
    end
    
    -- extract string for x and y
    xStr = string.sub(key, 1, dashIndex - 1)
    yStr = string.sub(key, dashIndex + 1, -1)
    x = tonumber(xStr)
    y = tonumber(yStr)
    
    return x, y
end
Helper.DoesRoomExist = function(roomKey)
    for k, _ in pairs(Rooms) do
        if (k == roomKey) then
            return true
        end
    end
    return false
end
Helper.InitializeRoom = function(roomKey)
    if (not Helper.DoesRoomExist(roomKey)) then
        Rooms[roomKey] = {}
        Rooms[roomKey].mDoors = {}
        Rooms[roomKey].mAnalyzed = false
        Rooms[roomKey].mNumDoors = 0
    end
end
Helper.InitializeDoor = function(roomKey, doorIndex)
    if (Helper.DoesRoomExist(roomKey)) then
        if (Rooms[roomKey].mDoors[doorIndex]) then
            Rooms[roomKey].mDoors[doorIndex] = {}
        end
    end
end
Helper.IsDoorKnown = function(roomKey, doorX, doorY)
    if (Helper.DoesRoomExist(roomKey)) then
        numDoors = Rooms[roomKey].mNumDoors
        for doorIndex = 1, numDoors do
            door = Rooms[roomKey].mDoors[doorIndex]
            if (door ~= nil) then
                if ((door.mX == doorX) and (door.mY == doorY)) then
                    return true
                end
            end
        end
    end
    return false
end

----------------------------
------ Hero Interface ------
----------------------------

SetupHero = function ()
   nret = cus.nameCharacter("Wanderer")
   Hero.mCurrentState = AnalyzeRoom
   Hero.mCurrentState.Init()
end
 
ExecuteHeroTurn = function ()
    Hero.mCurrentState.Run()
    per.think("Hola")
end